/*
 * ============================================================================
 * (C) Copyright IBM Corporation 2007, 2008.  All Rights Reserved.
 * 
 * You may only copy, modify, and distribute these samples internally.
 * These samples have not been tested under all conditions and are provided
 * to you by IBM without obligation of support of any kind.
 *
 * IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
 * CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
 * CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
 * ============================================================================
 */
package sample.kicker;

import java.util.HashMap;

import zero.core.context.GlobalContext;
import zero.kicker.Kicker;

/**
 * <p>The <code>SampleKicker</code> shows how to write a class that periodically
 * tests a resource and notifies a configured receiver when a message is present.</p>
 */
public class SampleKicker {

    /**
     * <p>Handler for <code>timer</code> event.</p>
     * 
     * <p>This implementation assumes that the event handler registration in the
     * <code>zero.config</code> file includes the following information in
     * the <code>instanceData</code>:<p>
     * 
     * <ul>
     * <li><code>queueName</code> the name of the monitored resource.</li>
     * <li><code>receiverURL</code> the URL for the corresponding receiver,
     * as required by <code>zero.kicker.Kicker</code>.</li>
     * <li>Optionally, other configuration for
     * <code>zero.kicker.Kicker</code> (see <code>zero.kicker</code>).</li>
     * </ul>
     */
    public void onTimer() {

        // Collect the queueName from the handler registration instanceData 
        String queueName = GlobalContext.zget("/event/queueName");
        
        // The timer task name, /event/_taskName, is our kicker name (see
        // zero.kicker)
        System.out.println(
                "SampleKicker.onTimer(): Timer event: kickerName=" +
                    GlobalContext.zget("/event/_taskName") +
                    ", queueName=" + queueName);

        // By default we don't use _immediateReschedule
        boolean immediateReschedule = false;

        // Test resource for new message
        if (hasMessages(queueName)) {
            System.out.println("SampleKicker.onTimer(): Queue has messages.");
            
            // We have a message so we must kick the receiver.
            immediateReschedule = kickReceiver(queueName);

            // If the receiver processes the message and returns true
            // then we use _immediateReschedule for the next timer event.
            // This ensures that messages can be processed as quickly as
            // possible, without waiting for the timer delay between each
            // one.
            
        } else {
            System.out.println("SampleKicker.onTimer(): Queue is empty.");
            
            // Nothing to do here
        }

        // Do we want to use _immediateReschedule?
        if (immediateReschedule) {
            System.out.println(
                    "SampleKicker.onTimer(): Message was processed. " +
                    "Using _immediateReschedule for next timer event.");
            
            // Signal _immediateReschedule for timer event (see zero.timer)
            GlobalContext.zput("/event/_immediateReschedule", true);
            
        } else {
            System.out.println(
                    "SampleKicker.onTimer(): Waiting until next timer period.");
            
            // Nothing to do here
        }
        
    }    

    /**
     * <p>Poll the resource to see if there are messages to process.</p>
     * 
     * @param queueName The name of the monitored resource.
     * @return <code>true</code> if the resource exists and contains messages.
     */
    private boolean hasMessages(String queueName) {
        
        // Connect to the queue and establish whether it has any messages
        // to process.
        SampleQueue queue = new SampleQueue(queueName);
        queue.connect();
        try {
            return queue.hasMessages();
        } finally {
            queue.disconnect();
        }
        
    }

    /**
     * <p>Notify the receiver that the resource has messages to process,
     * using the <code>zero.kicker.Kicker</code> API.</p>
     * 
     * @param queueName The name of the monitored resource.
     * @return <code>true</code> if receiver successfully processed the kick.
     */
    private boolean kickReceiver(String queueName) {
        
        // Build the map of data to be sent in the notification to the
        // receiver. This sample just needs to include the resource name.
        // The Kicker API adds additional information, such as the kicker name.
        HashMap<String, String> kickData = new HashMap<String, String>();
        kickData.put("queueName", queueName);

        // Send notification to receiver. The Kicker API looks in the timer
        // event context for the receiver URL and other information and
        // manages the process of retrying if the receiver cannot be contacted.
        // The result is true if a successful response was obtained from the
        // receiver.
        return Kicker.kick(kickData); 
        
    }
    
}
