/*
 * ============================================================================
 * (C) Copyright IBM Corporation 2007, 2008.  All Rights Reserved.
 * 
 * You may only copy, modify, and distribute these samples internally.
 * These samples have not been tested under all conditions and are provided
 * to you by IBM without obligation of support of any kind.
 *
 * IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
 * CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
 * CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
 * ============================================================================
 */
package sample.kicker;

import java.util.ArrayList;
import java.util.List;

import zero.core.context.GlobalContext;

/**
 * <p>The <code>SampleQueue</code> class models a simple queue resource for
 * the kicker and receiver demonstration.</p>
 * 
 * <p>As would normally be the case in a real deployment, a
 * <code>SampleQueue</code> instance does not logically own the queue but
 * provides a means by which the application can connect to and access
 * the queue resource.</p>
 * 
 * <p>Note that since the queue data is held in the global context
 * <code>/app</code> zone, the kicker and receiver that access it must reside
 * in the sample application process. In a real deployment, the queue resource
 * normally resides in an external location, such as a data base, and the
 * kicker and receiver can be deployed in separate applications.</p>
 */
public class SampleQueue {
    
    private String queueName;
    private List<String> queue;

    /**
     * <p>Constructor for <code>SampleQueue</code> instance.</p>
     * 
     * <p>A <code>SampleQueue</code> instance does not contain the queue but
     * provides the means by which the application accesses the queue
     * resource.</p>
     * 
     * @param queueName Name of queue to access.
     */
    public SampleQueue(Object queueName) {
        this.queueName = queueName.toString();
    }

    /*
     * (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "SampleQueue{" + queueName + "}";
    }
    
    /**
     * <p>Connect to queue resource, before any access can be made. </p>
     *
     * <p>In this implementation, we lock the queue data in the global
     * context and get a local copy.</p>
     */
    public void connect() {
        GlobalContext.zpost("/app#lock", "/sample/resource/" + queueName);
        queue = GlobalContext.zget("/app/sample/resource/" + queueName);
    }

    /**
     * <p>Commit changes made so far.</p>
     * 
     * <p>In this implementation, we copy our local copy into the
     * global context.</p>
     */
    public void commit() {
        GlobalContext.zput("/app/sample/resource/" + queueName, queue);
    }

    /**
     * <p>Disconnect from queue resource, throwing away any uncommitted
     * changes.</p>
     * 
     * <p>In this implementation, we just unlock the queue data in the
     * global context and forget our local copy.</p>
     */
    public void disconnect() {
        GlobalContext.zpost("/app#unlock", "/sample/resource/" + queueName);
        queue = null;
    }

    /**
     * <p>Test queue for messages to process.</p>
     * 
     * @return <code>true</code> if the queue contains messages.
     */
    public boolean hasMessages() {
        return (queue != null) && !(queue.isEmpty());
    }

    /**
     * <p>Get the number of messages on the queue.</p>
     * 
     * @return Number of messages remaining on the queue.
     */
    public int getLength() {
        return (queue == null) ? 0 : queue.size();
    }

    /**
     * <p>Read the next message from the queue.</p>
     * 
     * <p>This is a destructive read but the master copy of the
     * queue is not updated until <code>commit()</code> is called.</p>
     * 
     * @return Next message, or <code>null</code> if none available.
     */
    public String readMessage() {
        if (hasMessages()) {
            return queue.remove(0);
        } else {
            return null;
        }
    }
    
    /**
     * <p>Post message to end of the queue</p>
     * 
     * <p>Note that the master copy of the queue is not updated until
     * <code>commit()</code> is called.</p>
     * 
     * @param message Text of new message.
     */
    public void postMessage(String message) {
        if (queue == null) {
            queue = new ArrayList<String>();
        }
        queue.add(message);        
    }

    /**
     * <p>Browse contents of queue.</p>
     * 
     * @return Contents of queue.
     */
    public List<String> browseMessages() {
        List<String> result = new ArrayList<String>();
        if (queue != null) {
            result.addAll(queue);
        }
        return result;
    }

}
