/*
 * ============================================================================
 * (C) Copyright IBM Corporation 2007, 2008.  All Rights Reserved.
 * 
 * You may only copy, modify, and distribute these samples internally.
 * These samples have not been tested under all conditions and are provided
 * to you by IBM without obligation of support of any kind.
 *
 * IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
 * CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
 * CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
 * ============================================================================
 */
package sample.kicker;

import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;

import zero.core.context.GlobalContext;
import zero.core.context.GlobalContextURIs;
import zero.core.events.EventEngine;
import zero.core.events.dispatcher.OneDispatcher;

/**
 * <p>The <code>SampleReceiver</code> shows how to write a class that responds to
 * notifications from kickers based on the <code>SampleKicker</code> class.</p>
 */
public class SampleReceiver {

    /**
     * <p>Handler for <code>POST</code> event that is the <em>kick</em> from
     * the <code>SampleKicker</code>.</p>
     */
    public void onPOST() {
        
        // Collect the details of the kick notification from the request parameters
        String kickerName = GlobalContext.zget(GlobalContextURIs.Request.params + "/kickerName", null);
        String appName = GlobalContext.zget(GlobalContextURIs.Request.params + "/appName", null);
        String queueName = GlobalContext.zget(GlobalContextURIs.Request.params + "/queueName", null);

        System.out.println("SampleReceiver.onPOST(): Kick received: kickerName=" + kickerName + ", queueName=" + queueName);
        boolean success;

        // Before we respond to the kick, check whether another request thread
        // is already processing the specified kicker and queue.
        //
        // This is necessary for queue resources that allow simultaneous access
        // if we want to preserve message order. For example, if time taken to
        // process an individual message exceeds the timer that the kicker waits
        // for the response to its HTTP POST, the kicker assumes an error
        // and sends another notification. This starts a second receiver
        // request thread, which might begin processing the next message on
        // the queue. If we do not serialize the receiver processing this might
        // result in the processing of the second message being carried out
        // before the first message has completed.
        
        String lockKey = "/sample/receiver/" + appName + "/" + kickerName;
        GlobalContext.zpost("/app#lock", lockKey);
        try {
            success = processOneMessage(appName, kickerName, queueName);
        } finally {
            GlobalContext.zpost("/app#unlock", lockKey);
        }
        
        // Signal success of failure to the kicker
        int statusCode = (success) ? 204 : 500;
        GlobalContext.zput(GlobalContextURIs.Request.status, statusCode);
        System.out.println("SampleReceiver.onPOST(): Responding to kicker with HTTP status code: " + statusCode);

    }
    
    /**
     * <p>This method attempts to read a single message from the queue and process it.</p>
     * 
     * @param appName Application name from kick notification.
     * @param kickerName Kicker name from kick notification.
     * @param queueName Queue name from kick notification.
     * @return <code>true</code> if message processing was successful. 
     */
    private boolean processOneMessage(String appName, String kickerName, String queueName) {
        
        // Connect to queue resource
        SampleQueue queue = new SampleQueue(queueName);
        queue.connect();

        boolean success = false;

        try {
            
            // Start local transaction and read message
            String message = queue.readMessage();
            System.out.println("SampleReceiver.processOneMessage(): Processing message: " + message);

            if (message == null) {
                
                // No message present. Single success to kicker so that it tests the
                // queue before sending another kick notification. If we signalled failure
                // then kicker would just kick again.
                success = true;
                
            } else {

                try {
                    
                    // Attempt to process the message read from the queue
                    success = fireEvent(appName, kickerName, queueName, message);
                    
                } finally {
                    
                    // If event processing was successful we commit the resource to confirm
                    // that the message has been consumed.
                    if (success) {
                        // Confirm that message has been consumed
                        queue.commit();
                    } else {
                        // Rollback resource here, if necessary
                    }
                    
                }

            }
            
        } finally {

            // Disconnect from queue resource
            queue.disconnect();
            
        }

        // Return success or failure
        return success;
    }

    /**
     * <p>Fires the <code>sampleMessage</code> event and ensures that it is
     * dispatched to a message handler.</p>
     * 
     * @param appName Application name from kick notification.
     * @param kickerName Kicker name from kick notification.
     * @param queueName Queue name from kick notification.
     * @param message Message that was read from the queue.
     * @return <code>true</code> if message processing was successful. 
     */
    private boolean fireEvent(String appName, String kickerName, String queueName, String message)  {
        boolean success;
        try {
            
            // Prepare the global context for the request.            
            // The "sampleMessage" event just has the message text in the /request zone. 
            GlobalContext.zput("/request/sample/message", message);

            // Prepare the data for the /event zone.
            Map<String, Object> eventData = new HashMap<String, Object>(1);
            eventData.put("appName", appName);
            eventData.put("kickerName", kickerName);
            eventData.put("queueName", queueName);
            
            // Fire the "sampleMessage" event using an instance of OneDispatcher.
            // This both ensures that only one handler will be executed to process
            // the message and that a FileNotFoundException will be thrown if
            // no handler can be found to process the message.
            EventEngine.fire("sampleMessage", eventData, new OneDispatcher());
            
            // If we get here then the a handler for the "sampleMessage" event
            // has completed successfully.
            success = true;
            
        } catch (FileNotFoundException e) {
            
            // OneDispatcher throws FileNotFoundException if no handler was found
            // to process the message.
            success = false;
            
        } catch (NoSuchMethodException e) {
            success = false;            
        }

        return success;
    }
    
}
